using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using SensorTechnology;
using System.Runtime.InteropServices;
namespace SampleCallback
{
	/// <summary>
	/// Form1
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button btnPreview;
		private System.IntPtr m_hCamera;

		private bool m_bStatusTransfer = false;
		private bool m_bStatusAVIFile = false;
		private System.Windows.Forms.Button btnSnap;
		private bool m_bStatusPreviewWnd = false;
		private System.Windows.Forms.Button btnMovie;
		private System.Windows.Forms.SaveFileDialog saveFileDialog1;
		private System.Windows.Forms.Button btnFPS;
		
		private FormSnapShot m_FormSnapShot;

		private uint m_dwPreviewGDICallbackNo = 0;
		private StCam.fStCamPreviewGDICallbackFunc m_GDICallbackDelegate;
		private CFPS m_FPS = new CFPS(100);

		public Form1()
		{
			InitializeComponent();

			m_GDICallbackDelegate  = new StCam.fStCamPreviewGDICallbackFunc(OnGDICallbackFunc);
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer Code

		private void InitializeComponent()
		{
			this.btnPreview = new System.Windows.Forms.Button();
			this.btnSnap = new System.Windows.Forms.Button();
			this.btnMovie = new System.Windows.Forms.Button();
			this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
			this.btnFPS = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// btnPreview
			// 
			this.btnPreview.Location = new System.Drawing.Point(8, 8);
			this.btnPreview.Name = "btnPreview";
			this.btnPreview.Size = new System.Drawing.Size(120, 32);
			this.btnPreview.TabIndex = 0;
			this.btnPreview.Text = "Preview";
			this.btnPreview.Click += new System.EventHandler(this.btnPreview_Click);
			// 
			// btnSnap
			// 
			this.btnSnap.Location = new System.Drawing.Point(8, 48);
			this.btnSnap.Name = "btnSnap";
			this.btnSnap.Size = new System.Drawing.Size(120, 32);
			this.btnSnap.TabIndex = 1;
			this.btnSnap.Text = "Snap";
			this.btnSnap.Click += new System.EventHandler(this.btnSnap_Click);
			// 
			// btnMovie
			// 
			this.btnMovie.Location = new System.Drawing.Point(8, 88);
			this.btnMovie.Name = "btnMovie";
			this.btnMovie.Size = new System.Drawing.Size(120, 32);
			this.btnMovie.TabIndex = 2;
			this.btnMovie.Text = "Movie";
			this.btnMovie.Click += new System.EventHandler(this.btnMovie_Click);
			// 
			// saveFileDialog1
			// 
			this.saveFileDialog1.DefaultExt = "avi";
			this.saveFileDialog1.Filter = "AVI File|*.avi|All Files|*.*";
			// 
			// btnFPS
			// 
			this.btnFPS.Location = new System.Drawing.Point(8, 128);
			this.btnFPS.Name = "btnFPS";
			this.btnFPS.Size = new System.Drawing.Size(120, 32);
			this.btnFPS.TabIndex = 3;
			this.btnFPS.Text = "FPS";
			this.btnFPS.Click += new System.EventHandler(this.btnFPS_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 12);
			this.ClientSize = new System.Drawing.Size(136, 165);
			this.Controls.Add(this.btnFPS);
			this.Controls.Add(this.btnMovie);
			this.Controls.Add(this.btnSnap);
			this.Controls.Add(this.btnPreview);
			this.Name = "Form1";
			this.Text = "Form1";
			this.Load += new System.EventHandler(this.Form1_Load);
			this.Closed += new System.EventHandler(this.Form1_Closed);
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
			m_hCamera = StCam.Open(0);
			if(m_hCamera == IntPtr.Zero) ShowErrorMsg(StCam.GetLastError(m_hCamera));
			if(!StCam.SetReceiveMsgWindow(m_hCamera, this.Handle))
				ShowErrorMsg(StCam.GetLastError(m_hCamera));
			StatusChanged();
		}

		private void Form1_Closed(object sender, System.EventArgs e)
		{
			if(m_hCamera != System.IntPtr.Zero) StCam.Close(m_hCamera);
		}

		private void btnPreview_Click(object sender, System.EventArgs e)
		{
			if(m_bStatusPreviewWnd && m_bStatusTransfer)
			{
				//Stop
				if(!StCam.StopTransfer(m_hCamera))
					ShowErrorMsg(StCam.GetLastError(m_hCamera));
			}
			else
			{
				//Start
				if(!StCam.CreatePreviewWindow(m_hCamera,"Preview",StCam.WS_OVERLAPPEDWINDOW | StCam.WS_VISIBLE,0,0,0,0,System.IntPtr.Zero,System.IntPtr.Zero,true))
					ShowErrorMsg(StCam.GetLastError(m_hCamera));
				if(!StCam.StartTransfer(m_hCamera))
					ShowErrorMsg(StCam.GetLastError(m_hCamera));
			}
		}
		private void StatusChanged()
		{
			if(m_bStatusPreviewWnd && m_bStatusTransfer)
				btnPreview.Text = "STOP";
			else
				btnPreview.Text = "START";

			btnMovie.Enabled = !m_bStatusAVIFile;

			if(m_dwPreviewGDICallbackNo == 0)
			{
				btnFPS.Text = "Show FPS";
			}
			else
			{
				btnFPS.Text = "Erase FPS";
			}
		}
		[System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand,Name="FullTrust")]
		protected override void WndProc(ref Message m)
		{
			switch(m.Msg)
			{
				case StCam.WM_STCAM_TRANSFER_START:
					m_bStatusTransfer = true;
					StatusChanged();
					break;
				case StCam.WM_STCAM_TRANSFER_FINISH:
					m_bStatusTransfer = false;
					StatusChanged();
					if((uint)m.LParam != 0) ShowErrorMsg((uint)m.LParam);	
					break;
				case StCam.WM_STCAM_AVI_FILE_START:
					m_bStatusAVIFile = true;
					StatusChanged();
					break;
				case StCam.WM_STCAM_AVI_FILE_FINISH:
					m_bStatusAVIFile = false;
					StatusChanged();
					if((uint)m.LParam != 0) ShowErrorMsg((uint)m.LParam);	
					break;
				case StCam.WM_STCAM_PREVIEW_WINDOW_CREATE:
					m_bStatusPreviewWnd = true;
					StatusChanged();
					break;
				case StCam.WM_STCAM_PREVIEW_WINDOW_CLOSE:
					m_bStatusPreviewWnd = false;
					StatusChanged();
					if((uint)m.LParam != 0) ShowErrorMsg((uint)m.LParam);	
					break;

				default:
					base.WndProc(ref m);
					break;
			}
		}
		
		[DllImport("kernel32.dll", CharSet=System.Runtime.InteropServices.CharSet.Auto)]
		private static extern int FormatMessage(int dwFlags, IntPtr lpSource,
			int dwMessageId, int dwLanguageId, System.Text.StringBuilder lpBuffer, int nSize, IntPtr Arguments);
		[DllImport("kernel32.dll", CharSet=System.Runtime.InteropServices.CharSet.Auto)]
		private static extern IntPtr LoadLibraryEx(String lpszLibFile, IntPtr hFile, int dwFlags);
		[DllImport("kernel32.dll")]
		private static extern int FreeLibrary(IntPtr hModule);

		private void ShowErrorMsg(uint dwErrorCode)
		{
			int FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000;
			int FORMAT_MESSAGE_FROM_HMODULE = 0x00000800;

			int DONT_RESOLVE_DLL_REFERENCES = 0x00000001;

			System.Text.StringBuilder strErrorMsg = new System.Text.StringBuilder(255);
			int dwFlags = FORMAT_MESSAGE_FROM_SYSTEM;
			IntPtr ptrlpSource = IntPtr.Zero;

			int iFoundErrMsg = FormatMessage(dwFlags, ptrlpSource, (int)dwErrorCode, 0, strErrorMsg, strErrorMsg.Capacity, IntPtr.Zero);
			if(iFoundErrMsg == 0)
			{
				ptrlpSource = LoadLibraryEx("StCamMsg.dll", IntPtr.Zero, DONT_RESOLVE_DLL_REFERENCES);
				dwFlags |= FORMAT_MESSAGE_FROM_HMODULE;

				if(!ptrlpSource.Equals(IntPtr.Zero))
				{
					iFoundErrMsg = FormatMessage(dwFlags, ptrlpSource, (int)dwErrorCode, 0, strErrorMsg, strErrorMsg.Capacity, IntPtr.Zero);
					FreeLibrary(ptrlpSource);
				}
			}

			System.Windows.Forms.MessageBox.Show(strErrorMsg.ToString(), "Error Message", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Exclamation);

		}

		private void btnSnap_Click(object sender, System.EventArgs e)
		{
			bool bReval = true;
			uint nLastErrorNo = 0;

			do
			{
				//Get Image Size
				uint dwReserved;
				ushort wScanMode;
				uint dwOffsetX;
				uint dwOffsetY;
				uint dwWidth;
				uint dwHeight;
				bReval = StCam.GetImageSize(m_hCamera, out dwReserved, out wScanMode, out dwOffsetX, out dwOffsetY, out dwWidth, out dwHeight);
				if(!bReval)
				{
					nLastErrorNo = StCam.GetLastError(m_hCamera);
					break;
				}

				//Get Preview Pixel Format
				uint dwPreviewPixelFormat;
				bReval = StCam.GetPreviewPixelFormat(m_hCamera, out dwPreviewPixelFormat);
				if(!bReval)
				{
					nLastErrorNo = StCam.GetLastError(m_hCamera);
					break;
				}

				//Allocate Memory
				uint nBufferSize = dwWidth * dwHeight;
				System.Drawing.Imaging.PixelFormat pixelFormat = System.Drawing.Imaging.PixelFormat.Format8bppIndexed;
				switch(dwPreviewPixelFormat)
				{
					case(StCam.STCAM_PIXEL_FORMAT_24_BGR):
						nBufferSize *= 3;
						pixelFormat = System.Drawing.Imaging.PixelFormat.Format24bppRgb;
						break;
					case(StCam.STCAM_PIXEL_FORMAT_32_BGR):
						nBufferSize *= 4;
						pixelFormat = System.Drawing.Imaging.PixelFormat.Format32bppRgb;
						break;
				}

				byte[] pbyteImageBuffer = new byte[nBufferSize];

				//Take Snap Shot
				uint dwNumberOfByteTrans;
				uint[] pdwFrameNo = new uint[1];
				uint dwMilliseconds = 100;
				System.Runtime.InteropServices.GCHandle gch = System.Runtime.InteropServices.GCHandle.Alloc(pbyteImageBuffer, System.Runtime.InteropServices.GCHandleType.Pinned);
				System.IntPtr ptr = gch.AddrOfPinnedObject();

				bReval = StCam.TakePreviewSnapShot(m_hCamera, ptr, nBufferSize, out dwNumberOfByteTrans, pdwFrameNo, dwMilliseconds);

				gch.Free();
				if(!bReval)
				{
					nLastErrorNo = StCam.GetLastError(m_hCamera);
					break;
				}

				if(m_FormSnapShot == null)
				{
					m_FormSnapShot = new FormSnapShot();
					m_FormSnapShot.Closed += new EventHandler(FormSnapShot_Closed);
				}

				m_FormSnapShot.bUpdateSnapShot(pbyteImageBuffer, (int)dwWidth, (int)dwHeight, pixelFormat);
				m_FormSnapShot.Show();
			}while(false);

			//Show Error Message
			if(!bReval)
			{
				ShowErrorMsg(nLastErrorNo);
			}
		}
		private void FormSnapShot_Closed(object sender, EventArgs e)
		{
			m_FormSnapShot = null;
		}

		private void btnMovie_Click(object sender, System.EventArgs e)
		{
			bool bReval = true;
			uint nLastErrorNo = 0;

			if(DialogResult.OK == saveFileDialog1.ShowDialog(this))
			{
				bReval = StCam.SaveAVI(m_hCamera, saveFileDialog1.FileName, StCam.STCAM_AVI_COMPRESSOR_MJPG, 300, IntPtr.Zero);
				if(!bReval)
				{
					nLastErrorNo = StCam.GetLastError(m_hCamera);
				}
			}

			//Show Error Message
			if(!bReval)
			{
				ShowErrorMsg(nLastErrorNo);
			}
		}
		public void OnGDICallbackFunc(IntPtr hDC, uint dwWidth, uint dwHeight, uint dwFrameNo, IntPtr lpContext, IntPtr lpReserved)
		{
			Graphics grfx = System.Drawing.Graphics.FromHdcInternal(hDC);
			Font fontString = new Font(FontFamily.GenericSerif, 12);
			SolidBrush brushString = new SolidBrush(Color.Black);

			m_FPS.Check();

			double dblFPS = m_FPS.GetFPS();
			grfx.DrawString(dblFPS.ToString("0.0") + "FPS", fontString, brushString, 0, 0);

			brushString.Dispose();
			fontString.Dispose();
			grfx.Dispose();
		}
		private void btnFPS_Click(object sender, System.EventArgs e)
		{
			bool bReval = true;
			uint nLastErrorNo = 0;

			if(m_dwPreviewGDICallbackNo == 0)
			{
				m_FPS.Reset();
				bReval = StCam.AddPreviewGDICallback(m_hCamera, m_GDICallbackDelegate, IntPtr.Zero, out m_dwPreviewGDICallbackNo);
				if(!bReval)
				{
					nLastErrorNo = StCam.GetLastError(m_hCamera);
				}
			}
			else
			{
				bReval = StCam.RemovePreviewGDICallback(m_hCamera, m_dwPreviewGDICallbackNo);
				if(!bReval)
				{
					nLastErrorNo = StCam.GetLastError(m_hCamera);
				}
				m_dwPreviewGDICallbackNo = 0;
			}
			//Show Error Message
			if(!bReval)
			{
				ShowErrorMsg(nLastErrorNo);
			}
		}

	}
}
